/* oi.p
 *
 */

{lib/protop.i}
{lib/dynscreen.i}
{lib/tt_screenlist.i}

define new global shared variable pt_altWidth     as character no-undo.
define new global shared variable pt_showInterest as logical   no-undo.

define temp-table tt_interesting
  field fullName   as character
  field ts         as datetime-tz
  field severity   as character
  field cmdKey     as character
  field screenName as character
.

define temp-table tt_oi
  field txt_row    as integer
  field txt_handle as handle
.

define variable f as handle no-undo.

/* setup "Of Interest" frame
 *
 */

create frame f.
assign
  f:row           = 4
  f:column        = ( if pt_altWidth = "80" then 60 else 140 )
  f:width-chars   = 30
  f:box           = no
  f:top-only      = false
  f:overlay       = true
  f:scrollable    = false
  f:name          = "ofInterest"
  f:private-data  = "Of Interest"
no-error.


subscribe to "highlight-warning" anywhere run-procedure "interesting".
subscribe to "highlight-alert"   anywhere run-procedure "interesting".
subscribe to "highlight-good"    anywhere run-procedure "interesting".
subscribe to "highlight-data"    anywhere run-procedure "interesting".

session:add-super-procedure( this-procedure ).

return.


procedure scanDCList:

  define variable q  as handle  no-undo.
  define variable b  as handle  no-undo.
  define variable f  as handle  no-undo.

  define variable n  as integer no-undo.
  define variable j  as integer no-undo.

  define variable ok as logical no-undo.

  define variable hiliteAttr as character no-undo.

  empty temp-table tt_interesting.

  for each tt_screenList where tt_screenlist.screenVisible = no:

    assign
      b = tt_screenList.bufferHandle
      q = tt_screenList.queryHandle
    .

    if not valid-handle( b ) then next.

    if valid-handle( q ) then
      q:query-close() no-error.

    q:set-buffers( b ) no-error.

    q:query-prepare( "for each " + b:name ) no-error.

    ok = q:query-open() no-error.

    if ok = no then next.

    n = b:num-fields.

    if not valid-handle( q ) then
      do:
        /* message "invalid handle q for" b:name. */
        next.
      end.

    ok = q:get-first() no-error.

    do while not q:query-off-end:

      do j = 1 to n:

        f = b:buffer-field( j ).

        if f:extent <> 0 then next.

        hilite(
          b,
          f:name,
          f:buffer-value,
          input-output hiliteAttr
        ).

      end.

      q:get-next() no-error.

    end.

  end.

  return.

end.


procedure interesting:

  define input parameter action       as character no-undo.
  define input parameter metricName   as character no-undo.
  define input parameter metricValue  as character no-undo.
  define input parameter operator     as character no-undo.
  define input parameter threshold    as character no-undo.
  define input parameter metricFilter as character no-undo.

/*
 * message substitute( "&1 &2: &3 &4 &5 [&6]", action, metricName, metricValue, operator, threshold, metricFilter ).
 * pause.
 */

  find tt_interesting where tt_interesting.fullName = metricName no-error.
  if not available tt_interesting then
    do:

      find tt_screenList where tt_screenList.ttName = entry( 1, metricName, "." ) no-error.

      if tt_screenList.screenVisible = yes then return.

      create tt_interesting.
      assign
        tt_interesting.fullName   = metricName
        tt_interesting.cmdKey     = tt_screenList.ptCmd
        tt_interesting.screenName = tt_screenList.screenName
      .
    end.

  assign
    tt_interesting.ts         = now
    tt_interesting.severity   = action
  .

  return.

end.


procedure ofInterest:

  define variable headerColor    as integer no-undo.
  define variable actionColor    as integer no-undo extent 10.

  define variable t as handle  no-undo.

  define variable i as integer no-undo.
  define variable j as integer no-undo.

  define variable actScrns as integer no-undo format ">>9".
  define variable visScrns as integer no-undo format ">>9".

  /* if pt_altWidth = "" then return. */

  define variable interestTitle  as character no-undo format "x(21)".
  define variable interestingTxt as character no-undo format "x(20)" extent 10.

  if pt_showInterest = no then
    do:
      f:visible = no.
      return.
    end.

  headerColor = getColorNum( "header" ).
  interestingTxt = "".

  /* evaluate all data collectors
   */

  run scanDCList.

  for each tt_interesting break by tt_interesting.screenName by tt_interesting.fullName:

    /* maybe check tt_screenList.screenVisible? (only show interesting but invisible screens?)
     *
     *   if tt_screenList.screenVisible = yes then next.
     *
     */

    if first-of( tt_interesting.screenName ) then
      do:
        i = i + 1.
        actionColor[i] = 0.
      end.

    if last-of( tt_interesting.fullName ) then
      do:

        if tt_interesting.severity = "highlight-alert" then actionColor[i] = getColorNum( "alert" ).

        if actionColor[i] = 0 and tt_interesting.severity = "highlight-warning" then actionColor[i] = getColorNum( "warning" ).

        if actionColor[i] > 0 then
          do:
            j = i.
            interestingTxt[i] = substitute( "&1 = &2", tt_interesting.cmdKey, tt_interesting.screenName ).
          end.

      end.

    if last-of( tt_interesting.screenName ) then	/* if something was at least a warning then count it, otherwise do not increment i	*/
      i = j.

    if i >= 10 then leave.

  end.

  if i <= 0 then
    do:
      f:visible = no.
      for each tt_oi:
        delete object tt_oi.txt_handle no-error.
        delete tt_oi.
      end.
    end.
   else
    do:

      find tt_oi where tt_oi.txt_row = 0 no-error.
      if not available tt_oi then
        do:
          create tt_oi.
          create text t.
          assign
            tt_oi.txt_row    = 0
            tt_oi.txt_handle = t
          .
        end.

      t = tt_oi.txt_handle.

      assign
        t:frame        = f
        t:name         = "Interesting Stuff"
        t:row          = 1
        t:col          = 1
        t:format       = "x(19)"		/* substitute( "x(&1)", length( interestingTxt[i] ))	*/
        t:screen-value = "Of Interest"
        t:dcolor       = getColorNum( "browseTitle" )
        t:fgcolor      = getColorNum( "browseTitle" )
      no-error.

      j = i.

      do i = 1 to j:

        find tt_oi where tt_oi.txt_row = i no-error.
        if not available tt_oi then
          do:
            create tt_oi.
            create text t.
            assign
              tt_oi.txt_row    = i
              tt_oi.txt_handle = t
            .
          end.

        t = tt_oi.txt_handle.

        assign
          t:frame        = f
          t:name         = substitute( "t&1", i )
          t:row          = i + 1
          t:col          = 2
          t:format       = "x(18)"		/* substitute( "x(&1)", length( interestingTxt[i] ))	*/
          t:screen-value = interestingTxt[i]
          t:dcolor       = actionColor[i]
          t:fgcolor      = actionColor[i]
        no-error.

      end.

      for each tt_oi where tt_oi.txt_row > j:
        delete object tt_oi.txt_handle no-error.
        delete tt_oi.
      end.

      assign
        f:height  = j + 1
        f:visible = yes
      .

    end.


  return.

end.
