/* cnxlist.p
 *
 * mbpro dbname -p util/cnxlist.p -param "emaillist" > /tmp/cnxlist.err 2>&1
 *
 * this utility makes no attempt to support Windows email at this time
 *
 */


{lib/protop.i}

define temp-table tt_who no-undo
  field xid         as integer
  field usrNum      as integer   format ">>>>9"     label "Usr#"
  field usrName     as character format "x(20)"     label "Name"
  field whoPID      as character format "x(8)"      label "PID"
  field usrType     as character format "x(8)"      label "Flags"
  field connectAge  as character format "x(13)"     label "     Duration"
  field usrIPAddr   as character format "x(20)"     label "Device"
  field usrServ     as integer   format ">>>>>>"    label "Server"
  field statTRX     as character format "x(18)"     label "TRX Stat"
  field usrBlk      as character format "x(4)"      label "Blkd"
  field lastUpd     as character format "x(8)"      label " CSC Age"
  field lineNum     as integer   format "->>>>"     label "Line#"
  field procName    as character format "x(29)"     label "Program Name"

  index xid-idx      is unique xid
  index usrNum-idx   is unique primary usrNum xid
.

define new global shared variable pt_rptdir as character no-undo.

define variable rptFileName  as character no-undo.
define variable emailList    as character no-undo.


procedure getRUsers:

  define variable connUDT    as integer no-undo.
  define variable numDays    as integer no-undo.
  define variable numSeconds as integer no-undo.
  define variable trxStr     as character no-undo.
  define variable cnxFlags   as character no-undo.

  define variable i as integer no-undo.

  for each dictdb._Connect no-lock where _Connect-usr <> ?:

    cnxFlags = connectFlags( _Connect-Id ).

    if index( cnxFlags, "@" ) > 0 or	/* server				*/
       index( cnxFlags, "H" ) > 0 or	/* helper (apw, biw, aiw, wdog...	*/
       index( cnxFlags, "L" ) > 0 or	/* login broker				*/
       index( cnxFlags, "O" ) > 0 then	/* other (rpl*, aimd...)		*/

      next.

    trxStr = "".
    if _Connect-Transid > 0 then
      do:
        find first dictdb._Trans no-lock where _Trans-UsrNum = _Connect-Usr no-error.
        if available _Trans then
          do:
            trxStr = trim( substring( _Trans-state, 1, 5 )).
            if _Trans-duration <> ? then
              trxStr = trxStr + string( integer( truncate( _Trans-duration / 86400, 0 )), ">>>" ) + " " + string( _Trans-duration, "hh:mm:ss" ).
          end.
      end.

    connUDT = abs( interval( now, datetime( fixDateString( _Connect-Time )), "seconds" )) no-error.

    i = i + 1.

    find tt_who where tt_who.xid = _Connect-id no-error.

    if not available tt_who then create tt_who.

    lastStatement( _Connect-Id, output tt_who.lineNum, output tt_who.procName, tt_who.lastUpd ).

    assign
      tt_who.xid        = _Connect-Id
      tt_who.usrNum     = _Connect-Usr
      tt_who.usrName    = _Connect-Name
      tt_who.whoPID     = string( _Connect-PID )
      tt_who.usrServ    = ( if _Connect-Server = ? then 0 else _Connect-Server )
      tt_who.usrIPAddr  = ( if _Connect-Device = "batch" then "" else _Connect-Device )
      numSeconds        = connUDT modulo 86400
      numDays           = (( connUDT - numSeconds ) / 86400 )
      tt_who.connectAge = substitute( "&1&3 &2", string( numDays, ">>>" ), string( numSeconds, "hh:mm:ss" ), ( if numDays > 0 then "d" else " " ))
      statTRX           = trxStr
      tt_who.usrType    = cnxFlags
      tt_who.usrBlk     = ( if _Connect-Transid = 0 then "" else "TRX " + ( if trim( _Connect-Wait ) <> "--" then _Connect-Wait else "" ))
    .

  end.

  return.

end.


/* main body
 */

if search( "lib/protoplib.p" ) <> ? or search( "lib/protoplib.r" ) <> ? then
    run lib/protoplib.p  persistent.					/* load protop function library				*/

if search( "lib/vstlib.p" ) <> ? or search( "lib/vstlib.r" ) <> ? then
    run lib/vstlib.p     persistent.					/* load vst library (isReplTarget()...)			*/

if search( "lib/protop-cfg.p" ) <> ? or search( "lib/protop-cfg.r" ) <> ? then
    run lib/protop-cfg.p persistent.					/* initialize protop environment                	*/

if pt_rptdir = "" or pt_rptdir = ? then pt_rptdir = ".".

rptFileName = substitute( "&1/&2.&3", pt_rptdir, "cnxlist", "rpt" ).

emailList = session:parameter.

run getRUsers.

/* report the results
 */

output to value( rptFileName ).

put "User Connections by Connection Age (descending)" string( now ) format "x(19)" to 160 skip.
put pdbname(1) format "x(60)" skip.
put skip(1).

for each tt_who by connectAge descending:

  display
    usrNum
    usrName
    whoPID
    usrType
    connectAge
    usrIPAddr
    usrServ
    statTRX
    usrBlk
    lastUpd
    lineNum
    procName
   with
    frame showWho
    width 160
    down
  .

  down with frame showWho.

end.

output close.

if emailList <> "" and emailList <> ? and opsys = "unix" then
  os-command value( substitute( 'cat &2 | mailx -s "&3 for &4" &1', emailList, rptFileName, "Connection Report", pdbname(1))).

quit.

